CON
  _CLKMODE = XTAL1 + PLL16X 'Set to ext low-speed crystal, 4x PLL
  _XINFREQ = 5_000_000 'Frequency on XIN pin is 5 MHz 

  Sz            = 5
  buffSz        = 1<<(Sz)
  Vref          = 5000          'milliVolts 
                 
OBJ
  TV     : "TV_wText_db"
  delay : "timing"    
    
VAR
  byte cog

  byte  CPin
  byte  DPin
  byte  CSPin
  long  Sample
  long  buffer[buffSz]



PUB Main | Avg, i, index

  TV.Start(20)
  'Declare Pin assignments             
  Start(0,1,2)
  'Start the PASM program loop for gathering Samples  
  cog := cognew(@entry,@CPin)      
     

  'Main loop for updating the current "Average" and displaying to screen
  repeat
    i := cnt    'Begin Sample Avg time                         
    Avg := 0

    'Gather buffSz number of samples
    repeat index from 1 to buffSz
      Avg += Sample
    'Divide Avg by the number of samples
    Avg >>= (Sz)                
    i := cnt-i  'End Sample Avg time        


    'Start displaying the data  
    TV.out(0)
    TV.out(13)
    TV.out(13)         
    TV.str(string("Voltage Reading:"))
    TV.out(13)
    
    TV.str(string("Avg:    "))  'Display the averaged samples
    TV.decnd(Vref*Avg/4096,1,3) 'Convert binary sample to a voltage reading
    TV.str(string("V  (%"))
    TV.bin(Vref*Avg/4096,12)    'Display in binary to see noise
    TV.str(string(")"))            
    TV.out(13)
    
    TV.str(string("Sample: "))  'Display the individual sample
    TV.decnd(Vref*Sample/4096,1,3)
    TV.str(string("V  (%"))
    TV.bin(Vref*Sample/4096,12)
    TV.str(string(")"))            
    TV.out(13)
                                
    TV.str(string("Samples/Sec:")) 'Display how many samples/sec the code can do                          
    TV.out(13)                     'This does NOT include TV update times
    TV.dec(Clkfreq/i)
    TV.out(13)
    TV.str(string("Sample Size:")) 'Display current samples/Avg    
    TV.out(13)    
    TV.dec(buffSz)                
    TV.ShowScreen
    waitcnt(cnt+clkfreq/20)  



'The basic pin assignment on startup
PUB Start(C, D, CS)
  CPin := C
  DPin := D
  CSPin := CS

PUB Stop
  if cog
    cogstop(cog~ -  1)


'This was the original spin function. It was pretty slow - only achieving about
'1ksps raw with no averaging or smoothing. The new PASM routine is extremely
'fast and actually exceeds the speed of the ADS7816 chip
PUB ReadValue : Value | i
  'Set direction of pins to outputs
  dira[Dpin]~                   ' make dpin input
  dira[CPin]~~                                                            
  dira[CSPin]~~      
  'High to low transition to start conversion
  outa[CSPin] := 1
  outa[CSPin] := 0

  
  'Mimic 3 clock pulses as per chip datasheet
  repeat 3
    outa[CPin] := 1
'    waitcnt(cnt + 1000)
    outa[CPin] := 0    
'    waitcnt(cnt + 1000)
  'Do the actual data transfer

  
  Value~
  repeat 12  'grab 12 bits from the pin
    outa[Cpin] := 1                                              
    Value := (Value << 1) | ina[Dpin]
    outa[Cpin] := 0                                              

          
  'Make ChipSel high to force the chip into shutdown to conserve power
  outa[CSPin] := 1



DAT
              org       0
              
entry         mov       dataptrs, par
              rdlong    dcount,dataptrs
              shl       CLm,dcount                      'mask for Clock pin
              add       dataptrs,#4
              rdlong    dcount,dataptrs
              shl       DIm,dcount                      'mask for Data pin
              add       dataptrs,#4
              rdlong    dcount,dataptrs
              shl       CSm,dcount                      'mask for Chip Select pin
              add       dataptrs, #4              

              or        dira, CLm                       'make Clock an output
              or        dira, CSm                       'make Chip Select an output
              andn      dira, DIm                       'make DataIn an input

:mainloop

              or        outa, CLm                       'Set Clock High
              or        outa, CSm                       'Set CS High
                                                        
              andn      outa, CSm                       'Transition CS low to start conversion...
              nop                                       'wait a while for conversion to finish...
              nop                                       '...
              nop                                       '...
              nop
              nop
              nop
              nop
              nop
              nop                            

              'Transition Clock 3 times
              andn      outa, CLm                       'Low
              nop
              nop
              nop
              or        outa, CLm                       'High
              nop
              nop
              nop
              andn      outa, CLm                       'Low
              nop
              nop
              nop
              or        outa, CLm                       'High
              nop
              nop
              nop
              andn      outa, CLm                       'Low(Next high is in :bitloop

              
              sub dcount, dcount                        'Set the number of bits to 12
              add dcount, #12
              sub       val, val                        'clear out val

                            
:bitloop
              or        outa, CLm                       'Set clock high                       
              nop
              nop
              nop
              nop
              nop
              nop
              nop

              test      DIm,ina             wc          'Get the data bit on DataPin          
              rcl       val,#1
              
              andn       outa, CLm                      'Set clock low
              nop
              nop
              nop
              nop
              nop
              nop
              nop
              djnz      dcount, #:bitloop               'Loop back to get next bit                              

              wrlong    val, dataptrs                   'Write the value to "Sample" register     
              'Start the conversion over
              jmp       #:mainloop              
              

              







CLm           long      1                               'bit mask for clock pin
DIm           long      1                               'bit mask for data from adc pin
CSm           long      1                               'bit mask for cs pin

val           long      1
temp          long      1

dataptrs      res       1                               'hub start address for data
dataptr       res       1                               'current address for data
dcount        res       1                               'bit count, also temp during startup


FIT 496                    